/** @format */

const path = require('path');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const isDev = process.env.NODE_ENV === 'development';
const {getCustomConfig} = require('./app');
const browserslist = require(path.join(process.cwd(), 'package.json')).browserslist || ['defaults'];

const config = {
    rules: [
        {
            test: {test: /\.(j|t)s(x)?$/},
            use: [
                {
                    loader: 'babel-loader',
                    options: {
                        cacheDirectory: true, // 开启缓存
                        presets: [
                            [
                                '@babel/preset-env',
                                {
                                    targets: browserslist,
                                },
                            ],
                            '@babel/preset-typescript',
                            [
                                '@babel/preset-react',
                                {
                                    development: process.env.BABEL_ENV === 'development',
                                },
                            ],
                        ],
                        plugins: [
                            '@babel/plugin-syntax-dynamic-import',
                            '@babel/plugin-proposal-optional-chaining',
                            [
                                '@babel/plugin-proposal-decorators',
                                {
                                    legacy: true,
                                },
                            ],
                            [
                                '@babel/plugin-proposal-class-properties',
                                {
                                    loose: true,
                                },
                            ],
                            [
                                '@babel/plugin-transform-runtime',
                                {
                                    corejs: {
                                        version: 3,
                                        proposals: true,
                                    },
                                    helpers: true,
                                    regenerator: true,
                                    useESModules: false,
                                },
                            ],
                            'react-hot-loader/babel',
                        ],
                    },
                },
            ],
            include: path.resolve(process.cwd(), 'src'),
        },
        // scss 文件并且包含.module.将开启css-module
        {
            test: {test: /\.scss$/i, and: [/\.module\./i]},
            use: [
                isDev ? 'style-loader' : MiniCssExtractPlugin.loader,
                {
                    loader: 'css-loader',
                    options: {
                        importLoaders: 2,
                        modules: {
                            localIdentName: '[folder]-[local]-[hash:base64:5]',
                        },
                        localsConvention: 'camelCase',
                    },
                },
                'postcss-loader',
                'sass-loader',
            ],
            include: path.resolve(process.cwd(), 'src'),
        },
        // scss 文件不包含.module.将不开启css-module
        {
            test: {test: /\.scss$/i, not: [/\.module\./i]},
            use: [
                isDev ? 'style-loader' : MiniCssExtractPlugin.loader,
                {
                    loader: 'css-loader',
                },
                'postcss-loader',
                'sass-loader',
            ],
            include: path.resolve(process.cwd(), 'src'),
        },

        // 只要less 文件的后缀并且包含.module.开启modules
        {
            test: {test: /\.less$/i, and: [/\.module\./i]},
            use: [
                isDev ? 'style-loader' : MiniCssExtractPlugin.loader,
                {
                    loader: 'css-loader',
                    options: {
                        importLoaders: 2,
                        modules: {
                            localIdentName: '[folder]-[local]-[hash:base64:5]',
                        },
                        localsConvention: 'camelCase',
                    },
                },
                'postcss-loader',
                'less-loader',
            ],
            include: path.resolve(process.cwd(), 'src'),
        },
        //只要less 文件的后缀不包含.module.不开启modules
        {
            test: {
                test: /\.less$/i,
                not: [/\.module\./i],
            },
            use: [
                isDev ? 'style-loader' : MiniCssExtractPlugin.loader,
                {
                    loader: 'css-loader',
                },
                'postcss-loader',
                'less-loader',
            ],
        },
        // 只要是css后缀得文件，并且包含.module.将开启modules
        {
            test: {
                test: /\.css$/i,
                and: [/\.module\./i],
            },
            use: [
                isDev ? 'style-loader' : MiniCssExtractPlugin.loader,
                {
                    loader: 'css-loader',
                    options: {
                        importLoaders: 2,
                        modules: {
                            localIdentName: '[folder]-[local]-[hash:base64:5]',
                        },
                        localsConvention: 'camelCase',
                    },
                },
                'postcss-loader',
            ],
            include: path.resolve(process.cwd(), 'src'),
        },
        // 只要是css后缀得文件，不包含.module.将不开启modules
        {
            test: {test: /\.css$/i, not: [/\.module\./i]},
            use: [
                isDev ? 'style-loader' : MiniCssExtractPlugin.loader,
                {
                    loader: 'css-loader',
                },
                'postcss-loader',
            ],
            include: path.resolve(process.cwd(), 'src'),
        },
        {
            test: {test: /\.(png|jpg|svg|gif)$/},
            use: [
                {
                    loader: 'url-loader',
                    options: {
                        limit: 1000,
                        name: '[name][hash:8].[ext]',
                        fallback: 'file-loader',
                        // // 将图片打包到该公共目录下
                        outputPath: 'assets',
                    },
                },
            ],
        },
    ],
};
const {css = []} = getCustomConfig();
/**
 * 移除对同一个test得处理
 */
const newConfigRules = config.rules.filter(element => {
    return (
        css.findIndex(item => {
            return (
                item.test.toString() === element.test.test.toString() ||
                item.test.test.toString() === element.test.test.toString()
            );
        }) === -1
    );
});
module.exports = {...config, rules: [...css, ...newConfigRules]};
